import fs from 'fs';
import path from 'path';
import { IConfiguration } from './../model/configuration';

export class ServiceConfiguration<T> {
  /**
   * Select default path for service configuraiton path in different environment
   * By default Backtrace expect to run service from root service directory
   * by using npm commands. In environment different than production
   * @ServiceConfiguration class will read configuration file from root dir.
   * Otherwise from default Backtrace configuration directory
   */

  public getProjectSettingsPath(serviceName: string): string {
    switch (process.env.NODE_ENV) {
      case 'production':
        return `/etc/backtrace/${serviceName}/${serviceName}.conf`;
      default:
        return path.join(process.cwd(), `${serviceName}.conf`);
    }
  }

  /**
   * Loads service configuration file
   */
  public loadConfiguration(configurationPath: string): IConfiguration<T> {
    if (!configurationPath) {
      throw new Error(`configurationPath is undefined`);
    }
    const config = JSON.parse(
      fs.readFileSync(configurationPath, 'utf8'),
    ) as IConfiguration<T>;

    config.isProduction = process.env.NODE_ENV === 'production';

    if (config.ssl) {
      if (config.ssl.key) {
        config.ssl.key = fs.readFileSync(config.ssl.key);
      }
      if (config.ssl.cert) {
        config.ssl.cert = fs.readFileSync(config.ssl.cert);
      }
    }

    return config;
  }
}
