import * as url from 'url';

/**
 * Backtrae universe utils
 */
export class UniverseHelper {
  /**
   * return Backtrace host url
   * @param urlString sample url to any Backtrace resource
   * Example:
   * https://yolo.sp.backtrace.io => https://yolo.sp.backtrace.io
   * https://sampleHostName.backtrace.io/blah/poop/doop => https://sampleHostName.backtrace.io
   */
  public static getBacktraceHostName(urlString: string): string | undefined {
    if (!urlString) {
      return undefined;
    }
    const obj = url.parse(urlString);
    return `${obj.protocol}//${obj.hostname}`;
  }

  /**
   * Get an universe name from Backtrace url.
   * @param urlString sample url to any Backtrace resource
   * Example:
   * https://yolo.sp.backtrace.io => yolo
   * https://sampleHostName.backtrace.io => sampleHostName
   */
  public static getBacktaceUniverseName(urlString: string): string | undefined {
    if (!urlString) {
      return undefined;
    }
    const hostNameIndexEnd = urlString.indexOf('backtrace.io/');
    if (hostNameIndexEnd === -1) {
      return undefined;
    }
    return urlString.includes('submit.sp.backtrace.io')
      ? urlString.substring(hostNameIndexEnd, urlString.indexOf('/', hostNameIndexEnd))
      : urlString.substring(urlString.indexOf('//') + 2, urlString.indexOf('.sp.backtrace.io'));
  }
}
