const https = require('https');
const axios = require('axios');
const { service } = require('./service-layer');

const httpsAgent = service.identityManager.descr.isOnPremise
  ? new https.Agent({ rejectUnauthorized: false })
  : https.globalAgent;

function makeRequest(coronerdURL, token, actions) {
  const prefix = coronerdURL.endsWith('/') ? '' : '/';
  const requestUrl = `${coronerdURL}${prefix}api/bpg`;

  return axios.post(
    requestUrl,
    { actions },
    {
      httpsAgent,
      headers: {
        'Content-Type': 'application/json',
        'X-Coroner-Token': token,
      },
    }
  );
}

function actionToKey(action) {
  return `${action.type}__${action.action}`;
}

function parseResult(responseResult) {
  let result = {};

  try {
    if (responseResult.text === 'success') {
      result.data = responseResult.result;
      result.errors = [];
    } else {
      result.data = [];
      result.errors = [responseResult.text];
    }
  } catch (err) {
    result.data = [];
    result.errors = ['unable to parse BPG response'];
  }

  return result;
}

function parseResponse(response) {
  let results = {};

  response.actions.forEach(function (action, index) {
    let key = actionToKey(action);
    let result = parseResult(response.results[index]);
    if (results[key]) {
      let existingData = results[key].data;
      let existingErrors = results[key].errors;
      results[key].data = [...existingData, ...result.data];
      results[key].errors = [...existingErrors, ...result.errors];
    } else {
      results[key] = parseResult(response.results[index]);
    }
  });

  return results;
}

module.exports = {
  makeRequest,
  parseResponse,
  actionToKey,
};
