const backtrace = require('backtrace-node');
const APP_VERSION = require('./getVersion')();
const { service } = require('./service-layer');

const config = service.configuration;

if (process.env.NODE_ENV === 'production' && config.backtrace) {
  backtrace.initialize({
    endpoint: config.backtrace.endpoint,
    token: config.backtrace.token,
  });
}

module.exports = (error, attributes = {}) => {
  if (process.env.NODE_ENV === 'production' && config.backtrace) {
    try {
      service._logger.error(error);

      const client = backtrace.getBacktraceClient();

      client
        .reportAsync(error, Object.assign(attributes, { version: APP_VERSION }))
        .then((_results) => service._logger.info('[ Report sent ]'))
        .catch((err) => {
          service._logger.warn(
            'Backtrace report unable to send. This may be due to incorrect configuration information. If problems persist please reach out to a Backtrace I/O representative'
          );
          service._logger.error(err);
        });
    } catch (err) {
      service._logger.warn(
        'Backtrace report unable to send. This may be due to incorrect configuration information. If problems persist please reach out to a Backtrace I/O representative'
      );
      service._logger.error(err);
    }
  } else if (process.env.NODE_ENV !== 'test') {
    service._logger.error(error);
  }
};
