const { makeRequest, parseResponse, actionToKey } = require('./BPG');

const USER_TYPE = 'configuration/users';

const SIGNUP_PENDING_TYPE = 'configuration/signup_pending';

const GET_USERS_ACTION = {
  action: 'get',
  type: USER_TYPE,
};

const GET_SIGNUP_PENDING_ACTION = {
  action: 'get',
  type: SIGNUP_PENDING_TYPE,
};

function getUsersAndSignUpPending(coronerdURL, token) {
  return makeRequest(coronerdURL, token, [
    GET_USERS_ACTION,
    GET_SIGNUP_PENDING_ACTION,
  ]).then(({ data }) => parseResponse(data));
}

function getUsersFromResponse(results) {
  return results[actionToKey(GET_USERS_ACTION)].data;
}

function getSignupPendingFromResponse(results) {
  return results[actionToKey(GET_SIGNUP_PENDING_ACTION)].data;
}

function getDeleteSignupPendingActions(signupPendings, user) {
  return signupPendings
    .filter(function (pending) {
      return pending.username === user.username || pending.email === user.email;
    })
    .map(function (pending) {
      return {
        action: 'delete',
        type: SIGNUP_PENDING_TYPE,
        object: {
          universe: user.universe,
          email: pending.email,
        },
      };
    });
}

function getCreateUserAction({ email, username, universe }) {
  return {
    action: 'create',
    type: USER_TYPE,
    object: {
      email,
      username,
      universe,
      method: 'saml',
      role: 'member',
      superuser: 0,
    },
  };
}

function getUniqueUsername(users, email) {
  const baseUsername = email.split('@')[0];
  let username = baseUsername;
  let iterateUsername = true;
  let i = 1;
  while (iterateUsername) {
    let idx = users.findIndex((user) => user.username == username);
    if (idx === -1) {
      iterateUsername = false;
    } else {
      username = baseUsername + i;
      i++;
    }
  }

  return username;
}

function createUser(response, coronerdURL, token, email, universe) {
  let users = getUsersFromResponse(response);
  let signupPendings = getSignupPendingFromResponse(response);
  let username = getUniqueUsername(users, email);
  let user = {
    username,
    email,
    universe,
  };
  let signUpPendingsActions = getDeleteSignupPendingActions(
    signupPendings,
    user
  );
  let createUserAction = getCreateUserAction(user);
  const createUserActionKey = actionToKey(createUserAction);
  return makeRequest(coronerdURL, token, [
    ...signUpPendingsActions,
    createUserAction,
  ])
    .then(({ data }) => parseResponse(data))
    .then((response) => {
      let createUserResults = response[createUserActionKey];
      if (
        createUserResults &&
        createUserResults.errors &&
        createUserResults.errors.length > 0
      ) {
        throw new Error(createUserResults.errors[0]);
      }
      return createUserResults.data;
    });
}

module.exports = {
  getUsersAndSignUpPending,
  createUser,
};
